; 
;
;  XML Parser library by Nmeth Gbor. 2005.12.21.
;
;  Definitions and constants
;
;  This material is free to use, adapt or rewrite for everyone.
;
;
; 


	include automat.inc
	includelib automat.lib

; Encoding definitions
		ENCODING_UNKNOWN		EQU 00h
		ENCODING_UTF8			EQU 01h
		ENCODING_UTF16_LSB		EQU 02h
		ENCODING_UTF16_MSB		EQU 03h

; Number of encapsulated tags
		XML_MAX_DEPTH_OF_TAGS	EQU 014h

; Number of attributes a start tag can have
		XML_MAX_NUM_OF_ATTRIBS	EQU 0014h

; The auxillary string limits the size of tags names, attribute names and values and the size of CDATA
		XML_AUX_STRING_SIZE		EQU 0800h

; Error codes
		XML_PARSE_ERROR		EQU 01h
		XML_TAGS_DONT_MATCH		EQU 02h
		XML_INTERNAL_ERROR		EQU 03h
		XML_ENCODING_NOT_SUPPORTED	EQU 04h


; 
;
; XML Parser class
;
; The class consists of properties and methods.
; The properties are contained by the XMLParser structure.
; The methods have a first paranmeter, a pointer to a XMLparser structure.
;
;
; 

XMLParser	STRUCT

; Properties
		encoding		dd 0		; type of character encoding, not used yet

		lpStartTagHandler	dd NULL		; pointer of Start Tag event handler
		lpCDATAHandler		dd NULL		; pointer of CDATA event handler		
		lpEndTagHandler		dd NULL		; pointer of End Tag event handler
		lpCommandHandler	dd NULL		; pointer of Command Name event handler

		lpCommand		dd NULL		; pointer to command processor

		dwErrorCode		dd 0		; last error code
		
	; array of pointers to tags
		lpTags			dd XML_MAX_DEPTH_OF_TAGS dup (0)
		dwCurrentLevel		dd 0		; current level in hierarchy (index of current tag)

	; array of pointer pairs: attrib name, attrib value
		lpAttribs		dd XML_MAX_NUM_OF_ATTRIBS dup (NULL,NULL)
					dd NULL		; end of attrib list
		dwCurrentAttrib		dd 0		; index of current attrib

	; auxillary string
		lpAuxString		dd NULL		; pointer to string
		lpAuxStringEnd		dd NULL		; pointer to end of string
		lpChar			dd NULL		; auxillary pointer to a char

		dwCurrentRow		dd 1		; row being processed
		dwCurrentCol		dd 1		; char being processed

		lpXMLstream 		dd NULL		; pointer to char of XML input being processed

		charParserEFSM		EFSM_CLASS <>	; automaton

; 
; include transition.asm				; states and transitions of charParserEFSM

	ST_START		EQU XMLParser.ST_Start
	ST_START1OPENTAG	EQU XMLParser.ST_Start1OpenTag
	ST_COMMANDNAME		EQU XMLParser.ST_CommandName
	ST_COMMAND		EQU XMLParser.ST_Command
	ST_COMMANDEND		EQU XMLParser.ST_CommandEnd
	ST_START2OPENTAG	EQU XMLParser.ST_Start2OpenTag
	ST_OPENTAG		EQU XMLParser.ST_OpenTag
	ST_ATTRIBSTART		EQU XMLParser.ST_AttribStart
	ST_ATTRIBNAME		EQU XMLParser.ST_AttribName
	ST_AFTERATTRIBNAME	EQU XMLParser.ST_AfterAttribName
	ST_ATTRIB1VALUE		EQU XMLParser.ST_Attrib1Value
	ST_ATTRIBVALUE		EQU XMLParser.ST_AttribValue
	ST_AFTERATTRIBVALUE	EQU XMLParser.ST_AfterAttribValue
	ST_EMPTYTAG		EQU XMLParser.ST_EmptyTag
	ST_AFTEROPENTAG		EQU XMLParser.ST_AfterOpenTag
	ST_AFTER2OPENTAG	EQU XMLParser.ST_After2OpenTag
	ST_STARTTAG		EQU XMLParser.ST_StartTag
	ST_CDATA		EQU XMLParser.ST_CData
	ST_AFTERCDATA		EQU XMLParser.ST_AfterCData
	ST_START1CLOSETAG	EQU XMLParser.ST_Start1CloseTag
	ST_START2CLOSETAG	EQU XMLParser.ST_Start2CloseTag
	ST_CLOSETAG		EQU XMLParser.ST_CloseTag
	ST_CLOSE2TAG		EQU XMLParser.ST_Close2Tag
	ST_AFTERCLOSETAG	EQU XMLParser.ST_AfterCloseTag
	ST_COMMENTSTART		EQU XMLParser.ST_CommentStart
	ST_COMMENT2START	EQU XMLParser.ST_Comment2Start
	ST_COMMENT		EQU XMLParser.ST_Comment
	ST_COMMENTEND		EQU XMLParser.ST_CommentEnd
	ST_COMMENT2END		EQU XMLParser.ST_Comment2End





	ST_Start			dd 2,NULL,NULL
		Start1			EFSMTransitionN <,ST_START1OPENTAG,NULL,'<'>
		Start2			EFSMTransitionE <,ST_START,NULL,<3,009h>>
					dd 00Ah,020h

	ST_Start1OpenTag		dd 4,NULL,NULL
		Start1OpenTag1		EFSMTransitionN <,ST_COMMENTSTART,NULL,'!'>
		Start1OpenTag2		EFSMTransitionN <,ST_COMMANDNAME,offset XML_CommandNameStart,'?'>
		Start1OpenTag3		EFSMTransitionN <,ST_START2OPENTAG,offset XML_OpenTagStart,'_'>
		Start1OpenTag4		EFSMTransitionI <,ST_OPENTAG,offset XML_OpenTagStart,040h,0FFh>

	ST_CommandName			dd 2,NULL,NULL
		CommandName1		EFSMTransitionI <,ST_COMMANDNAME,offset XML_CommandName,040h,0FFh>
		CommandName2		EFSMTransitionE <,ST_COMMAND,offset XML_CommandNameEnd,<3,009h>>
					dd 00Ah,020h

	ST_Command			dd 2,NULL,NULL
		Command1		EFSMTransitionN <,ST_COMMANDEND,offset XML_Command,'?'>
		Command2		EFSMTransitionI <,ST_COMMAND,offset XML_Command,001h,0FFh>

	ST_CommandEnd			dd 2,NULL,NULL
		CommandEnd1		EFSMTransitionN <,ST_AFTERCLOSETAG,offset XML_CommandEnd,'>'>
		CommandEnd2		EFSMTransitionI <,ST_COMMAND,offset XML_Command,001h,0FFh>

	ST_Start2OpenTag		dd 2,NULL,NULL
		Start2OpenTag1		EFSMTransitionI <,ST_OPENTAG,offset XML_OpenTag,030h,039h>
		Start2OpenTag2		EFSMTransitionI <,ST_OPENTAG,offset XML_OpenTag,040h,0FFh>

	ST_OpenTag			dd 4,NULL,NULL
		OpenTag1		EFSMTransitionN <,ST_AFTEROPENTAG,offset XML_OpenTagEndAtOnce,'>'>
		OpenTag2		EFSMTransitionI <,ST_OPENTAG,offset XML_OpenTag,030h,039h>
		OpenTag3		EFSMTransitionI <,ST_OPENTAG,offset XML_OpenTag,040h,0FFh>
		OpenTag4		EFSMTransitionE <,ST_ATTRIBSTART,offset XML_OpenTagName,<3,009h>>
					dd 00Ah,020h

	ST_AttribStart			dd 4,NULL,NULL
		AttribStart1		EFSMTransitionN <,ST_EMPTYTAG,NULL,'/'>
		AttribStart2		EFSMTransitionN <,ST_AFTEROPENTAG,offset XML_OpenTagEnd,'>'>
		AttribStart3		EFSMTransitionI <,ST_ATTRIBNAME,offset XML_AttribName,040h,0FFh>
		AttribStart4		EFSMTransitionE <,ST_ATTRIBSTART,NULL,<3,009h>>
					dd 00Ah,020h

	ST_AttribName			dd 3,NULL,NULL
		AttribName1		EFSMTransitionN <,ST_ATTRIB1VALUE,offset XML_AttribNameEnd,'='>
		AttribName2		EFSMTransitionI <,ST_ATTRIBNAME,offset XML_AttribName,041h,0FFh>
		AttribName3		EFSMTransitionE <,ST_AFTERATTRIBNAME,offset XML_AttribNameEnd,<3,009h>>
					dd 00Ah,020h

	ST_AfterAttribName		dd 2,NULL,NULL
		AfterAttribName1	EFSMTransitionN <,ST_ATTRIB1VALUE,NULL,'='>
		AfterAttribName2	EFSMTransitionE <,ST_AFTERATTRIBNAME,NULL,<3,009h>>
					dd 00Ah,020h

	ST_Attrib1Value			dd 2,NULL,NULL
		Attrib1Value1		EFSMTransitionN <,ST_ATTRIBVALUE,NULL,'"'>
		Attrib1Value2		EFSMTransitionE <,ST_ATTRIB1VALUE,NULL,<3,009h>>
					dd 00Ah,020h

	ST_AttribValue			dd 2,NULL,NULL
		AttribValue1		EFSMTransitionN <,ST_AFTERATTRIBVALUE,offset XML_AttribValueEnd,'"'>
		AttribValue2		EFSMTransitionI <,ST_ATTRIBVALUE,offset XML_AttribValue,001h,0FFh>

	ST_AfterAttribValue		dd 4,NULL,NULL
		AfterAttribValue1	EFSMTransitionN <,ST_EMPTYTAG,NULL,'/'>
		AfterAttribValue2	EFSMTransitionN <,ST_AFTEROPENTAG,offset XML_OpenTagEnd,'>'>
		AfterAttribValue3	EFSMTransitionI <,ST_ATTRIBNAME,NULL,040h,0FFh>
		AfterAttribValue4	EFSMTransitionE <,ST_ATTRIBSTART,NULL,<3,009h>>
					dd 00Ah,020h

	ST_EmptyTag			dd 2,NULL,NULL		
		EmptyTag1		EFSMTransitionN <,ST_AFTEROPENTAG,offset XML_EmptyTag,'>'>
		EmptyTag2		EFSMTransitionE <,ST_EMPTYTAG,NULL,<3,009h>>
					dd 00Ah,020h

	ST_AfterOpenTag			dd 3,NULL,NULL
		AfterOpenTag1		EFSMTransitionN <,ST_STARTTAG,NULL,'<'>
		AfterOpenTag2		EFSMTransitionI <,ST_CDATA,offset XML_CDataStart,021h,0FFh>
		AfterOpenTag3		EFSMTransitionE <,ST_AFTER2OPENTAG,offset XML_CDataStart,<3,009h>>
					dd 00Ah,020h

	ST_After2OpenTag		dd 3,NULL,NULL
		After2OpenTag1		EFSMTransitionN <,ST_STARTTAG,NULL,'<'>
		After2OpenTag2		EFSMTransitionI <,ST_CDATA,offset XML_CData,021h,0FFh>
		After2OpenTag3		EFSMTransitionE <,ST_AFTER2OPENTAG,offset XML_CData,<3,009h>>
					dd 00Ah,020h

	ST_StartTag			dd 5,NULL,NULL
		StartTag1		EFSMTransitionN <,ST_COMMENTSTART,NULL,'!'>
		StartTag2		EFSMTransitionI <,ST_OPENTAG,offset XML_OpenTagStart,040h,0FFh>
		StartTag3		EFSMTransitionN <,ST_START2OPENTAG,offset XML_OpenTagStart,'_'>
		StartTag4		EFSMTransitionN <,ST_START1CLOSETAG,NULL,'/'>
		StartTag5		EFSMTransitionE <,ST_STARTTAG,NULL,<3,009h>>
					dd 00Ah,020h

	ST_CData			dd 3,NULL,NULL
		CData1			EFSMTransitionN <,ST_AFTERCDATA,offset XML_CDataEnd,'<'>
		CData2			EFSMTransitionI <,ST_CDATA,offset XML_CData,021h,0FFh>
		CData3			EFSMTransitionE <,ST_CDATA,offset XML_CData,<3,009h>>
					dd 00Ah,020h

	ST_AfterCData			dd 1,NULL,NULL
		AfterCData1		EFSMTransitionN <,ST_START1CLOSETAG,NULL,'/'>

	ST_Start1CloseTag		dd 2,NULL,NULL
		Start1CloseTag1		EFSMTransitionN <,ST_START2CLOSETAG,offset XML_CloseTagStart,'_'>
		Start1CloseTag2		EFSMTransitionI <,ST_CLOSETAG,offset XML_CloseTagStart,040h,0FFh>

	ST_Start2CloseTag		dd 2,NULL,NULL
		Start2CloseTag1		EFSMTransitionI <,ST_CLOSETAG,offset XML_CloseTag,030h,039h>
		Start2CloseTag2		EFSMTransitionI <,ST_CLOSETAG,offset XML_CloseTag,040h,0FFh>

	ST_CloseTag			dd 4,NULL,NULL
		CloseTag1		EFSMTransitionN <,ST_AFTERCLOSETAG,offset XML_CloseTagEnd,'>'>
		CloseTag2		EFSMTransitionI <,ST_CLOSETAG,offset XML_CloseTag,030h,039h>
		CloseTag3		EFSMTransitionI <,ST_CLOSETAG,offset XML_CloseTag,040h,0FFh>
		CloseTag4		EFSMTransitionE <,ST_CLOSE2TAG,offset XML_CloseTagEnd,<3,009h>>
					dd 00Ah,020h

	ST_Close2Tag			dd 2,NULL,NULL
		Close2Tag1		EFSMTransitionN <,ST_AFTERCLOSETAG,NULL,'>'>
		Close2Tag2		EFSMTransitionE <,ST_CLOSE2TAG,NULL,<3,009h>>
					dd 00Ah,020h

	ST_AfterCloseTag		dd 2,NULL,NULL
		AfterCloseTag1		EFSMTransitionE <,ST_AFTERCLOSETAG,NULL,<3,009h>>
					dd 00Ah,020h
		AfterCloseTag2		EFSMTransitionN <,ST_STARTTAG,NULL,'<'>

	ST_CommentStart			dd 1,NULL,NULL
		CommentStart1		EFSMTransitionN <,ST_COMMENT2START,NULL,'-'>

	ST_Comment2Start		dd 1,NULL,NULL
		Comment2Start1		EFSMTransitionN <,ST_COMMENT,NULL,'-'>

	ST_Comment			dd 3,NULL,NULL
		Comment1		EFSMTransitionN <,ST_COMMENTEND,NULL,'-'>
		Comment2		EFSMTransitionI <,ST_COMMENT,NULL,021h,0FFh>
		Comment3		EFSMTransitionE <,ST_COMMENT,NULL,<3,009h>>
					dd 00Ah,020h

	ST_CommentEnd			dd 3,NULL,NULL
		CommentEnd1		EFSMTransitionN <,ST_COMMENT2END,NULL,'-'>
		CommentEnd2		EFSMTransitionI <,ST_COMMENT,NULL,020h,0FFh>
		CommentEnd3		EFSMTransitionE <,ST_COMMENT,NULL,<3,009h>>
					dd 00Ah,020h

	ST_Comment2End			dd 3,NULL,NULL
		Comment2End1		EFSMTransitionN <,ST_AFTERCLOSETAG,NULL,'>'>
		Comment2End2		EFSMTransitionI <,ST_COMMENT,NULL,020h,0FFh>
		Comment2End3		EFSMTransitionE <,ST_COMMENT,NULL,<3,009h>>
					dd 00Ah,020h

; 

    
XMLParser	ENDS

; Methods

; 
;
; lpParser XML_createParser(lpStartHandler:DWORD,lpCDATAHandler:DWORD,lpEndHandler:DWORD,lpCommandHandler:DWORD)
;
; Creates and initializes an instant/object of the XMLParser class.
; - lpStartHandler is a pointer of a callback function that must have the following form: void MyStartTagHandler(lpTag:DWORD,lpAttributeList:DWORD)
;   On error the handler must return with CF set, that will cause XML_PARSE_ERROR.
; - lpCDATAHandler is a pointer of a callback function that must have the following form: void MyCDATAHandler(lpCDATA:DWORD)
; - lpEndHandler is a pointer of a callback function that must have the following form: void MyEndTagHandler(lpTag:DWORD)
; - lpCommandHandler is a pointer of a callback function that must have the following form: void MyCommandHandler(lpCommandTag:DWORD)
; The lpTag,lpCDATA and lpCommandTag are pointers to unicode strings.
; The lpAttribute is a pointer to an array of pointer pairs. Both pointers are unicode string pointers,the first points to an attribute's name, the second points to the
; attribute's value.
; The handlers can have NULL as input value, in this case the corresponding event won't be handled.
;
; Returns in eax the address of created XMLParser on success or NULL on fail
;
; 

	XML_createParser PROTO :DWORD,:DWORD,:DWORD,:DWORD

; 
;
; void XML_destroyParser(lpParser:DWORD)
;
; Destroys the lpParser object, frees all allocated memory space.
; Has no return value.
;
; 

	XML_destroyParser PROTO :DWORD

; 
;
; XML_resetParser(lpParser:DWORD)
;
; 

	XML_resetParser PROTO :DWORD

; 
;
; XML_setHandlers(lpParser:DWORD,lpStHandler:DWORD,lpCDHandler:DWORD,lpEndHandler:DWORD,lpCommandHandler:DWORD)
;
; 

	XML_setHandlers PROTO :DWORD,:DWORD,:DWORD,:DWORD,:DWORD

; 
;
; XML_error(lpParser:DWORD)
;
; 
	
	XML_error PROTO :DWORD

; 
;
; XML_parse(lpParser:DWORD,lpData:DWORD,dwBytes:DWORD)
;
; 
	
	XML_parse PROTO :DWORD,:DWORD,:DWORD

; 
;
; Start tag handler(DWORD lpTAG,DWORD lpAttributeList)
;
;
; CData handler(DWORD lpDATA)
;
;
; End tag handler(DWORD lpTAG)
;
;
; Command handler(DWORD lpCommandName)
;
;
; 
