; 
;
; AUTOMATON includes by Nemeth Gabor/nemeth_g@freemail.hu (2005.10.26.)
;
; 


IFNDEF __AUTOMAT

	__AUTOMAT	EQU <AUTOMATON_LIBRARY>

	ENUM		STRUCT

			Count	dd ?
			List	dd ?

	ENUM		ENDS

; 
;
; Codes of input matching rules
;
; 

	SM_Normal		EQU	00h
	SM_Interval		EQU	01h
	SM_IntervalWithEx	EQU	02h
	SM_Enum			EQU	03h

; 
;
; The transition structures
;
; 


	NumOfTransitions		EQU 0
	EnterState			EQU 4
	ExitState			EQU 8


	FSMTransition	STRUCT

			dwStateIn	dd ?
			dwInput		dd ?
			dwStateOut	dd ?

	FSMTransition	ENDS

; 
;
; The extended FSM does not have a State property, it is substituted by the TransTable property.
; Two states that have the same transitions must not be different states!
;
; 
;
; EFSM transition types
;
; 
; Normal
EFSMTransitionN		STRUCT

		; ----   common in every transition type  ----
			dwMatchType	dd SM_Normal
			dwStateOut	dd ?
			lpAction	dd ?
		; ----  ---- ---- ---- ---- ---- ---- ---- ---- ----
			dwInput		dd ?

EFSMTransitionN		ENDS

; 
; Interval
EFSMTransitionI		STRUCT

		; ----   common in every transition type  ----
			dwMatchType	dd SM_Interval
			dwStateOut	dd ?
			lpAction	dd ?
		; ----  ---- ---- ---- ---- ---- ---- ---- ---- ----
			dwLower		dd ?
			dwUpper		dd ?

EFSMTransitionI		ENDS

; 
; Interval with exception
EFSMTransitionIE	STRUCT

		; ----   common in every transition type  ----
			dwMatchType	dd SM_IntervalWithEx
			dwStateOut	dd ?
			lpAction	dd ?
		; ----  ---- ---- ---- ---- ---- ---- ---- ---- ----
			dwLower		dd ?
			dwUpper		dd ?

; exception list, variable length
			objList		ENUM <>	

EFSMTransitionIE	ENDS

; 
; Enum
EFSMTransitionE		STRUCT

		; ----   common in every transition type  ----
			dwMatchType	dd SM_Enum
			dwStateOut	dd ?
			lpAction	dd ?
		; ----  ---- ---- ---- ---- ---- ---- ---- ---- ----
; this field is variable length...
			objList		ENUM <>	

EFSMTransitionE		ENDS

; 
;
; The automaton class to simulate a FSM
;
; Structure of transition table:
;  0000h: count of transitions
;  0004h- FSMTransition structures
;
; 

FSM_CLASS	STRUCT
; Properties
		dwState			dd ?		; state property
		lpTransitionTable	dd ?		; pointer to transition table
		lpAcceptStates		dd ?		; pointer to array of accept states
		lpNextState		dd ?		; pointer to 'Next State' function
							; just for easy usage
FSM_CLASS	ENDS
; 
; Methods
	
    FSM_Create		PROTO :DWORD,:DWORD,:DWORD,:DWORD
;   - pointer to FSM structure
;   - initial state
;   - initial transition table address
;   - list of accept states

    FSM_Reset		PROTO :DWORD,:DWORD
;   - initial state
;   - initial transition table address

    FSM_NextState	PROTO
;   - esi:FSM structure address
;   - eax: input symbol

    FSM_CheckState	PROTO
;   - esi:FSM structure address


; 
;
; The automaton class to simulate an EFSM
;
; Structure of transition table:
;  0000h: count of transitions
;  0004h: pointer of EnterState action
;  0008h: pointer of ExitState action
;  000Ch- EFSMTransition structures
;
; -------------------------------------------------------------------------
EFSM_CLASS      STRUCT
; Properties
		lpTransitionTable	dd ?		; pointer to transition table
		lpAcceptStates		dd ?		; pointer to array of accept states
		lpNextState		dd ?		; pointer to 'Next State' function
						; just for easy usage
EFSM_CLASS      ENDS
; 
; Methods

    EFSM_Create		PROTO :DWORD,:DWORD
;   - initial transition table address
;   - list of accept states

    EFSM_Init		PROTO :DWORD,:DWORD,:DWORD
;   - pointer to EFSM structure
;   - initial transition table address
;   - list of accept states

    EFSM_Reset		PROTO :DWORD,:DWORD
;   - pointer to EFSM structure
;   - initial state

    EFSM_NextState	PROTO :DWORD,:DWORD
;   - esi:EFSM structure address
;   - eax: input symbol

    EFSM_CheckState	PROTO :DWORD
;   - esi:EFSM structure address

ENDIF